__author__ = 'DarkWeb'

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup


#parses description pages, so takes html pages of description pages using soup object, and parses it for info it needs
#stores info it needs in different lists, these lists are returned after being organized
#@param: soup object looking at html page of description page
#return: 'row' that contains a variety of lists that each hold info on the description page
def torbay_description_parser(soup):

    # Fields to be parsed
    vendor = "-1"  # 0 *Vendor_Name
    success = "-1"  # 1 Vendor_Successful_Transactions
    rating_vendor = "-1"  # 2 Vendor_Rating
    name = "-1"  # 3 *Product_Name
    describe = "-1"  # 4 Product_Description
    CVE = "-1"  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures) dont worry about that much
    MS = "-1"  # 6 Product_MS_Classification (Microsoft Security) dont worry about that much
    category = "-1"  # 7 Product_Category
    views = "-1"  # 8 Product_Number_Of_Views
    reviews = "-1"  # 9 Product_Number_Of_Reviews
    rating_item = "-1"  # 10 Product_Rating
    addDate = "-1"  # 11 Product_AddedDate
    BTC = "-1"  # 12 Product_BTC_SellingPrice
    USD = "-1"  # 13 Product_USD_SellingPrice
    EURO = "-1"  # 14 Product_EURO_SellingPrice
    sold = "-1"  # 15 Product_QuantitySold
    left = "-1"  # 16 Product_QuantityLeft
    shipFrom = "-1"  # 17 Product_ShippedFrom
    shipTo = "-1"  # 18 Product_ShippedTo

    # Finding Product Name
    name = soup.find('div', {'class': 'product-information'}).find('h1').text.strip()

    # Finding Vendor
    vendor = soup.find('div', {"class": "profile-info"}).find('a').text.strip()

    # Finding Vendor Rating
    rating_vendor.append(-1)

    # Finding Successful Transactions
    success.append(-1)

    bae = soup.find('div', {'class': "box"}).find_all('ul')

    # Finding Prices
    USD = soup.find('div', {'class': "total-price"}).find('span').text.strip()


    # Finding the Product Category
    category = soup.find('div', {'class': "profile-info"}).find('p').find('a').text.strip()

    # Finding the Product Quantity Available
    left.append(-1)

    # Finding Number Sold
    sold.append(-1)

    li = bae[3].find_all('li')

    # Finding Shipment Information (Origin)
    if "Ships from:" in li[-2].text:
        shipFrom = li[-2].text
        shipFrom = shipFrom.replace("Ships from: ", "")
        # shipFrom = shipFrom.replace(",", "")
        shipFrom = shipFrom.strip()

    # Finding Shipment Information (Destination)
    shipTo = li[-1].find('div', {'title': "List of countries is scrollable"}).text
    shipTo = shipTo.replace("Ships to: ", "")
    shipTo = shipTo.strip()
    if "certain countries" in shipTo:
        countries = ""
        tags = li[-1].find_all('span', {'class': "tag"})
        for tag in tags:
            country = tag.text.strip()
            countries += country + ", "
        shipTo = countries.strip(", ")

    # Finding the Product description
    describe = soup.find('div', {'class': "pre-line"}).text
    describe = describe.replace("\n", " ")
    describe = describe.strip()

    '''# Finding the Number of Product Reviews
    tag = soup.findAll(text=re.compile('Reviews'))
    for index in tag:
        reviews = index
        par = reviews.find('(')
        if par >=0:
            reviews = reviews.replace("Reviews (","")
            reviews = reviews.replace(")","")
            reviews = reviews.split(",")
            review = str(abs(int(reviews[0])) + abs(int(reviews[1])))
        else  :
            review = "-1"'''

    # Searching for CVE and MS categories
    cve = soup.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
    if cve:
        CVE = " "
        for idx in cve:
            CVE += (idx)
            CVE += "  "
            CVE = CVE.replace(',', ' ')
            CVE = CVE.replace('\n', '')
    ms = soup.findAll(text=re.compile('MS\d{2}-\d{3}'))
    if ms:
        MS = " "
        for im in ms:
            MS += (im)
            MS += " "
            MS = MS.replace(',', ' ')
            MS = MS.replace('\n', '')

    # Populating the final variable (this should be a list with all fields scraped)
    row = (vendor, rating_vendor, success, name, describe, CVE, MS, category, views, reviews, rating_item, addDate,
           BTC, USD, EURO, sold, left, shipFrom, shipTo)

    # Sending the results
    return row


#parses listing pages, so takes html pages of listing pages using soup object, and parses it for info it needs
#stores info it needs in different lists, these lists are returned after being organized
#@param: soup object looking at html page of listing page
#return: 'row' that contains a variety of lists that each hold info on the listing page
def torbay_listing_parser(soup):

    # Fields to be parsed
    nm = 0  # *Total_Products (Should be Integer)
    mktName = "TorBay"  # 0 *Marketplace_Name
    vendor = []  # 1 *Vendor y
    rating_vendor = []  # 2 Vendor_Rating
    success = []  # 3 Vendor_Successful_Transactions
    name = []  # 4 *Product_Name y
    CVE = []  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures) dont worry about this
    MS = []  # 6 Product_MS_Classification (Microsoft Security) dont worry about this
    category = []  # 7 Product_Category y
    describe = []  # 8 Product_Description
    views = []  # 9 Product_Number_Of_Views
    reviews = []  # 10 Product_Number_Of_Reviews
    rating_item = []  # 11 Product_Rating
    addDate = []  # 12 Product_AddDate
    BTC = []  # 13 Product_BTC_SellingPrice
    USD = []  # 14 Product_USD_SellingPrice y
    EURO = []  # 15 Product_EURO_SellingPrice
    sold = []  # 16 Product_QuantitySold
    qLeft = []  # 17 Product_QuantityLeft
    shipFrom = []  # 18 Product_ShippedFrom
    shipTo = []  # 19 Product_ShippedTo
    href = []  # 20 Product_Links

    listing = soup.findAll('div', {"class": "product-card"})

    # Populating the Number of Products
    nm = len(listing)

    for a in listing:
        bae = a.findAll('a', href=True)

        # Adding the url to the list of urls
        link = bae[0].get('href')
        link = cleanLink(link)
        href.append(link)

        # Finding the Product
        product = bae[1].find('p').text
        product = product.replace('\n', ' ')
        product = product.replace(",", "")
        product = product.replace("...", "")
        product = product.strip()
        name.append(product)

        bae = a.find('div', {'class': "media-content"}).find('div').find_all('div')

        if len(bae) >= 5:
            # Finding Prices
            price = bae[0].text
            ud = price.replace(" USD", " ")
            # u = ud.replace("$","")
            u = ud.replace(",", "")
            u = u.strip()
            USD.append(u)
            # bc = (prc[1]).strip(' BTC')
            # BTC.append(bc)

            # Finding the Vendor
            vendor_name = bae[1].find('a').text
            vendor_name = vendor_name.replace(",", "")
            vendor_name = vendor_name.strip()
            vendor.append(vendor_name)

            # Finding the Category
            cat = bae[2].find('small').text
            cat = cat.replace("Category: ", "")
            cat = cat.replace(",", "")
            cat = cat.strip()
            category.append(cat)

            # Finding Number Sold and Quantity Left
            num = bae[3].text
            num = num.replace("Sold: ", "")
            num = num.strip()
            sold.append(num)

            quant = bae[4].find('small').text
            quant = quant.replace("In stock: ", "")
            quant = quant.strip()
            qLeft.append(quant)

            # Finding Successful Transactions
            freq = bae[1].text
            freq = freq.replace(vendor_name, "")
            freq = re.sub(r'Vendor Level \d+', "", freq)
            freq = freq.replace("(", "")
            freq = freq.replace(")", "")
            freq = freq.strip()
            success.append(freq)

        # Searching for CVE and MS categories
        cve = a.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
        if not cve:
            cveValue="-1"
        else:
            cee = " "
            for idx in cve:
                cee += (idx)
                cee += "  "
                cee = cee.replace(',', ' ')
                cee = cee.replace('\n', '')
            cveValue=cee
        CVE.append(cveValue)
        
        ms = a.findAll(text=re.compile('MS\d{2}-\d{3}'))
        if not ms:
            MSValue="-1"
        else:
            me = " "
            for im in ms:
                me += (im)
                me += " "
                me = me.replace(',', ' ')
                me = me.replace('\n', '')
            MSValue=me
        MS.append(MSValue)

    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(mktName, nm, vendor, rating_vendor, success, name, CVE, MS, category, describe, views,
                            reviews, rating_item, addDate, BTC, USD, EURO, sold, qLeft, shipFrom, shipTo, href)

#called by the crawler to get description links on a listing page
#@param: beautifulsoup object that is using the correct html page (listing page)
#return: list of description links from a listing page
def torbay_links_parser(soup):

    # Returning all links that should be visited by the Crawler

    href = []
    listing = soup.find('section', {"id": "content"}).findAll('div', {"class": "product-card"})

    for a in listing:
        bae = a.find('div', {"class": "pc-footer"}).find('a', {"class": "btn btn-primary"}, href=True)
        link = bae['href']
        href.append(link)

    return href